@@ -1,15 +1,18 @@
 #[macro_use]
 extern crate clap;
 extern crate rayon;
+extern crate regex;
 extern crate unicode_width;
 
 use std::collections::HashSet;
+use std::process;
 
 use self::display::draw_it;
 use clap::{App, AppSettings, Arg};
 use dir_walker::walk_it;
 use dir_walker::WalkData;
-use filter::{get_biggest, get_by_depth};
+use filter::{get_all_file_types, get_biggest, get_by_depth};
+use regex::Regex;
 use std::cmp::max;
 use std::path::PathBuf;
 use terminal_size::{terminal_size, Height, Width};
@@ -61,6 +64,7 @@ fn get_height_of_terminal() -> usize {
     }
 }
 
+#[cfg(windows)]
 fn get_width_of_terminal() -> usize {
     // Windows CI runners detect a very low terminal width
     if let Some((Width(w), Height(_h))) = terminal_size() {
@@ -70,6 +74,28 @@ fn get_width_of_terminal() -> usize {
     }
 }
 
+#[cfg(not(windows))]
+fn get_width_of_terminal() -> usize {
+    if let Some((Width(w), Height(_h))) = terminal_size() {
+        w as usize
+    } else {
+        DEFAULT_TERMINAL_WIDTH
+    }
+}
+
+fn get_regex_value(maybe_value: Option<&str>) -> Option<Regex> {
+    match maybe_value {
+        Some(v) => match Regex::new(v) {
+            Ok(r) => Some(r),
+            Err(e) => {
+                eprintln!("Ignoring bad value for regex {:?}", e);
+                process::exit(1);
+            }
+        },
+        None => None,
+    }
+}
+
 fn main() {
     let default_height = get_height_of_terminal();
     let def_num_str = default_height.to_string();
@@ -148,9 +174,39 @@ fn main() {
         .arg(
             Arg::with_name("ignore_hidden")
                 .short("i") // Do not use 'h' this is used by 'help'
-                .long("ignore_hidden")
+                .long("ignore_hidden") //TODO: fix change - -> _
                 .help("Do not display hidden files"),
         )
+        .arg(
+            Arg::with_name("invert_filter")
+                .short("v")
+                .long("invert-filter")
+                .takes_value(true)
+                .number_of_values(1)
+                .multiple(true)
+                .conflicts_with("filter")
+                .conflicts_with("types")
+                .conflicts_with("depth")
+                .help("Exclude files matching this regex. To ignore png files type: -v \"\\.png$\" "),
+        )
+        .arg(
+            Arg::with_name("filter")
+                .short("e")
+                .long("filter")
+                .takes_value(true)
+                .number_of_values(1)
+                .multiple(true)
+                .conflicts_with("types")
+                .conflicts_with("depth")
+                .help("Only include files matching this regex. For png files type: -e \"\\.png$\" "),
+        )
+        .arg(
+            Arg::with_name("types")
+                .short("t")
+                .long("file_types")
+                .conflicts_with("depth")
+                .help("show only these file types"),
+        )
         .arg(
             Arg::with_name("width")
                 .short("w")
@@ -159,15 +215,18 @@ fn main() {
                 .number_of_values(1)
                 .help("Specify width of output overriding the auto detection of terminal width"),
         )
-        .arg(Arg::with_name("inputs").multiple(true))
+        .arg(Arg::with_name("inputs").multiple(true).default_value("."))
         .get_matches();
 
-    let target_dirs = {
-        match options.values_of("inputs") {
-            None => vec!["."],
-            Some(r) => r.collect(),
-        }
-    };
+    let target_dirs = options
+        .values_of("inputs")
+        .expect("Should be a default value here")
+        .collect();
+
+    let summarize_file_types = options.is_present("types");
+
+    let maybe_filter = get_regex_value(options.value_of("filter"));
+    let maybe_invert_filter = get_regex_value(options.value_of("invert_filter"));
 
     let number_of_lines = match value_t!(options.value_of("number_of_lines"), usize) {
         Ok(v) => v,
@@ -217,23 +276,36 @@ fn main() {
 
     let walk_data = WalkData {
         ignore_directories: ignored_full_path,
+        filter_regex: maybe_filter,
+        invert_filter_regex: maybe_invert_filter,
         allowed_filesystems,
         use_apparent_size,
         by_filecount,
         ignore_hidden,
     };
 
-    let (nodes, errors) = walk_it(simplified_dirs, walk_data);
+    let (top_level_nodes, has_errors) = walk_it(simplified_dirs, walk_data);
 
     let tree = {
-        match depth {
-            None => get_biggest(nodes, number_of_lines),
-            Some(depth) => get_by_depth(nodes, depth),
+        match (depth, summarize_file_types) {
+            (_, true) => get_all_file_types(top_level_nodes, number_of_lines),
+            (Some(depth), _) => get_by_depth(top_level_nodes, depth),
+            (_, _) => get_biggest(
+                top_level_nodes,
+                number_of_lines,
+                options.values_of("filter").is_some()
+                    || options.value_of("invert_filter").is_some(),
+            ),
         }
     };
 
+    if options.is_present("filter") {
+        println!("Filtering by: {}", options.value_of("filter").unwrap());
+    }
+    if has_errors {
+        eprintln!("Did not have permissions for all directories");
+    }
     draw_it(
-        errors,
         options.is_present("display_full_paths"),
         !options.is_present("reverse"),
         no_colors,
