@@ -1,4 +1,5 @@
 use assert_cmd::Command;
+use std::ffi::OsStr;
 use std::str;
 use std::sync::Once;
 
@@ -37,7 +38,7 @@ fn copy_test_data(dir: &str) {
     };
 }
 
-pub fn initialize() {
+fn initialize() {
     INIT.call_once(|| {
         copy_test_data("tests/test_dir");
         copy_test_data("tests/test_dir2");
@@ -45,160 +46,101 @@ pub fn initialize() {
     });
 }
 
+fn exact_output_test<T: AsRef<OsStr>>(valid_outputs: Vec<String>, command_args: Vec<T>) {
+    initialize();
+
+    let mut a = &mut Command::cargo_bin("dust").unwrap();
+    for p in command_args {
+        a = a.arg(p);
+    }
+    let output: String = str::from_utf8(&a.unwrap().stdout).unwrap().into();
+
+    assert!(valid_outputs
+        .iter()
+        .fold(false, |sum, i| sum || output.contains(i)));
+}
+
 // "windows" result data can vary by host (size seems to be variable by one byte); fix code vs test and re-enable
 #[cfg_attr(target_os = "windows", ignore)]
 #[test]
 pub fn test_main_basic() {
     // -c is no color mode - This makes testing much simpler
-    initialize();
-    let mut cmd = Command::cargo_bin("dust").unwrap();
-    let assert = cmd.arg("-c").arg("/tmp/test_dir/").unwrap().stdout;
-    let output = str::from_utf8(&assert).unwrap();
-    assert!(output.contains(&main_output()));
+    exact_output_test(main_output(), vec!["-c", "/tmp/test_dir/"])
 }
 
 #[cfg_attr(target_os = "windows", ignore)]
 #[test]
 pub fn test_main_multi_arg() {
-    initialize();
-    let mut cmd = Command::cargo_bin("dust").unwrap();
-    let assert = cmd
-        .arg("-c")
-        .arg("/tmp/test_dir/many/")
-        .arg("/tmp/test_dir")
-        .arg("/tmp/test_dir")
-        .unwrap()
-        .stdout;
-    let output = str::from_utf8(&assert).unwrap();
-    assert!(output.contains(&main_output()));
+    let command_args = vec![
+        "-c",
+        "/tmp/test_dir/many/",
+        "/tmp/test_dir",
+        "/tmp/test_dir",
+    ];
+    exact_output_test(main_output(), command_args);
 }
 
-#[cfg(target_os = "macos")]
-fn main_output() -> String {
-    r#"
+fn main_output() -> Vec<String> {
+    // Some linux currently thought to be Manjaro, Arch
+    // Although probably depends on how drive is formatted
+    let mac_and_some_linux = r#"
    0B     ┌── a_file    │░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░█ │   0%
  4.0K     ├── hello_file│████████████████████████████████████████████████ │ 100%
  4.0K   ┌─┴ many        │████████████████████████████████████████████████ │ 100%
  4.0K ┌─┴ test_dir      │████████████████████████████████████████████████ │ 100%
  "#
     .trim()
-    .to_string()
-}
+    .to_string();
 
-#[cfg(target_os = "linux")]
-fn main_output() -> String {
-    r#"
+    let ubuntu = r#"
    0B     ┌── a_file    │               ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░█ │   0%
  4.0K     ├── hello_file│               ░░░░░░░░░░░░░░░░█████████████████ │  33%
  8.0K   ┌─┴ many        │               █████████████████████████████████ │  67%
   12K ┌─┴ test_dir      │████████████████████████████████████████████████ │ 100%
   "#
     .trim()
-    .to_string()
-}
+    .to_string();
 
-#[cfg(target_os = "windows")]
-fn main_output() -> String {
-    "windows results vary by host".to_string()
+    vec![mac_and_some_linux, ubuntu]
 }
 
 #[cfg_attr(target_os = "windows", ignore)]
 #[test]
 pub fn test_main_long_paths() {
-    initialize();
-    let mut cmd = Command::cargo_bin("dust").unwrap();
-    let assert = cmd
-        .arg("-c")
-        .arg("-p")
-        .arg("/tmp/test_dir/")
-        .unwrap()
-        .stdout;
-    let output = str::from_utf8(&assert).unwrap();
-    assert!(output.contains(&main_output_long_paths()));
+    let command_args = vec!["-c", "-p", "/tmp/test_dir/"];
+    exact_output_test(main_output_long_paths(), command_args);
 }
 
-#[cfg(target_os = "macos")]
-fn main_output_long_paths() -> String {
-    r#"
+fn main_output_long_paths() -> Vec<String> {
+    let mac_and_some_linux = r#"
    0B     ┌── /tmp/test_dir/many/a_file    │░░░░░░░░░░░░░░░░░░░░░░░░░░░░█ │   0%
  4.0K     ├── /tmp/test_dir/many/hello_file│█████████████████████████████ │ 100%
  4.0K   ┌─┴ /tmp/test_dir/many             │█████████████████████████████ │ 100%
  4.0K ┌─┴ /tmp/test_dir                    │█████████████████████████████ │ 100%
 "#
     .trim()
-    .to_string()
-}
-
-#[cfg(target_os = "linux")]
-fn main_output_long_paths() -> String {
-    r#"   
+    .to_string();
+    let ubuntu = r#"
    0B     ┌── /tmp/test_dir/many/a_file    │         ░░░░░░░░░░░░░░░░░░░█ │   0%
  4.0K     ├── /tmp/test_dir/many/hello_file│         ░░░░░░░░░░██████████ │  33%
  8.0K   ┌─┴ /tmp/test_dir/many             │         ████████████████████ │  67%
   12K ┌─┴ /tmp/test_dir                    │█████████████████████████████ │ 100%
 "#
     .trim()
-    .to_string()
-}
-
-#[cfg(target_os = "windows")]
-fn main_output_long_paths() -> String {
-    "windows results vary by host".to_string()
-}
-
-#[cfg_attr(target_os = "windows", ignore)]
-#[test]
-pub fn test_apparent_size() {
-    initialize();
-    let mut cmd = Command::cargo_bin("dust").unwrap();
-    let assert = cmd.arg("-c").arg("-s").arg("/tmp/test_dir").unwrap().stdout;
-    let output = str::from_utf8(&assert).unwrap();
-    assert!(output.contains(&output_apparent_size()));
-}
-
-#[cfg(target_os = "linux")]
-fn output_apparent_size() -> String {
-    r#"
-   0B     ┌── a_file    │                       ░░░░░░░░░░░░░░░░░░░░░░░░█ │   0%
-   6B     ├── hello_file│                       ░░░░░░░░░░░░░░░░░░░░░░░░█ │   0%
- 4.0K   ┌─┴ many        │                       █████████████████████████ │  50%
- 8.0K ┌─┴ test_dir      │████████████████████████████████████████████████ │ 100%     
-"#
-    .trim()
-    .to_string()
-}
-
-#[cfg(target_os = "macos")]
-fn output_apparent_size() -> String {
-    r#"
-   0B     ┌── a_file    │                    ░░░░░░░░░░░░░░░░░░░░░░░░░░░█ │   0%
-   6B     ├── hello_file│                    ░░░░░░░░░░░░░░░░░░░░░░░░░░██ │   3%
- 134B   ┌─┴ many        │                    ████████████████████████████ │  58%
- 230B ┌─┴ test_dir      │████████████████████████████████████████████████ │ 100%
-"#
-    .trim()
-    .to_string()
-}
-
-#[cfg(target_os = "windows")]
-fn output_apparent_size() -> String {
-    "windows results vary by host".to_string()
+    .to_string();
+    vec![mac_and_some_linux, ubuntu]
 }
 
 // Check against directories and files whos names are substrings of each other
 #[cfg_attr(target_os = "windows", ignore)]
 #[test]
 pub fn test_substring_of_names_and_long_names() {
-    initialize();
-    let mut cmd = Command::cargo_bin("dust").unwrap();
-    let output = cmd.arg("-c").arg("/tmp/test_dir2").unwrap().stdout;
-    let output = str::from_utf8(&output).unwrap();
-    assert!(output.contains(&no_substring_of_names_output()));
+    let command_args = vec!["-c", "/tmp/test_dir2"];
+    exact_output_test(no_substring_of_names_output(), command_args);
 }
 
-#[cfg(target_os = "linux")]
-fn no_substring_of_names_output() -> String {
-    "
+fn no_substring_of_names_output() -> Vec<String> {
+    let ubuntu = "
    0B   ┌── long_dir_name_what_a_very_long_dir_name_what_happens_when_this_g..
  4.0K   ├── dir_name_clash
  4.0K   │ ┌── hello
@@ -208,12 +150,9 @@ fn no_substring_of_names_output() -> String {
   24K ┌─┴ test_dir2
     "
     .trim()
-    .into()
-}
+    .into();
 
-#[cfg(target_os = "macos")]
-fn no_substring_of_names_output() -> String {
-    "
+    let mac_and_some_linux = "
    0B   ┌── long_dir_name_what_a_very_long_dir_name_what_happens_when_this_g..
  4.0K   │ ┌── hello
  4.0K   ├─┴ dir
@@ -223,48 +162,33 @@ fn no_substring_of_names_output() -> String {
   12K ┌─┴ test_dir2
   "
     .trim()
-    .into()
-}
-
-#[cfg(target_os = "windows")]
-fn no_substring_of_names_output() -> String {
-    "PRs".into()
+    .into();
+    vec![mac_and_some_linux, ubuntu]
 }
 
 #[cfg_attr(target_os = "windows", ignore)]
 #[test]
 pub fn test_unicode_directories() {
-    initialize();
-    let mut cmd = Command::cargo_bin("dust").unwrap();
-    let output = cmd.arg("-c").arg("/tmp/test_dir_unicode").unwrap().stdout;
-    let output = str::from_utf8(&output).unwrap();
-    assert!(output.contains(&unicode_dir()));
+    let command_args = vec!["-c", "/tmp/test_dir_unicode"];
+    exact_output_test(unicode_dir(), command_args);
 }
 
-#[cfg(target_os = "linux")]
-fn unicode_dir() -> String {
+fn unicode_dir() -> Vec<String> {
     // The way unicode & asian characters are rendered on the terminal should make this line up
-    "
+    let ubuntu = "
    0B   ┌── ラウトは難しいです！.japan│                                 █ │   0%
    0B   ├── 👩.unicode                │                                 █ │   0%
  4.0K ┌─┴ test_dir_unicode            │██████████████████████████████████ │ 100%
     "
     .trim()
-    .into()
-}
+    .into();
 
-#[cfg(target_os = "macos")]
-fn unicode_dir() -> String {
-    "
+    let mac_and_some_linux = "
    0B   ┌── ラウトは難しいです！.japan│                                 █ │   0%
    0B   ├── 👩.unicode                │                                 █ │   0%
    0B ┌─┴ test_dir_unicode            │                                 █ │   0%
     "
     .trim()
-    .into()
-}
-
-#[cfg(target_os = "windows")]
-fn unicode_dir() -> String {
-    "".into()
+    .into();
+    vec![mac_and_some_linux, ubuntu]
 }
